#ifndef ACOUSTIC_MATERIAL_H_INCLUDED
#define ACOUSTIC_MATERIAL_H_INCLUDED

#include <cmath>
#include <string>

class acoustic_material
{
public:
    acoustic_material(std::string const &name, double rho, double kappaP0)
        : m_name(name), m_rho(rho), m_kappa_P0(kappaP0)
    {
    }

    std::string const &get_name() const { return m_name; }
    double get_rho() const { return m_rho; }
    double get_kappa_P0() const { return m_kappa_P0; }
    double get_speed_of_sound() const { return std::sqrt(m_kappa_P0 / m_rho); }
    double get_specific_impedance() const { return m_rho * get_speed_of_sound(); }

private:
    std::string m_name;
    double m_rho, m_kappa_P0;
};

#endif
