#ifndef CFS_DATA_INTERFACE_H_INCLUDED
#define CFS_DATA_INTERFACE_H_INCLUDED

#include <map>
#include <string>
#include <vector>

#include "h5_file.hpp"

namespace NiHu
{

namespace internal
{

	template <class scalar_t>
	class cfs_data_reader;

	template <class scalar_t>
	class cfs_data_reader<std::complex<scalar_t>>
	{
	public:
		typedef Eigen::Matrix<std::complex<scalar_t>, Eigen::Dynamic, Eigen::Dynamic> matrix_t;

		// as this is the matrix whose buffer is loaded from the hdf file, this must be stored in rowmajor
		typedef Eigen::Matrix<scalar_t, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor> real_matrix_t;

		static matrix_t read(h5_file &data_file, std::string const &path)
		{
			real_matrix_t dr = data_file.read<real_matrix_t>(path + "/Real");
			real_matrix_t di = data_file.read<real_matrix_t>(path + "/Imag");
			return dr + std::complex<scalar_t>(0., 1.) * di;
		}
	};
}

template <class scalar_t>
class cfs_data_interface
{
public:
	typedef Eigen::Matrix<scalar_t, Eigen::Dynamic, Eigen::Dynamic> matrix_t;

	cfs_data_interface() = default;

	void read(h5_file &file, std::string const &path)
	{
		std::vector<std::string> region_names = file.get_subgroup_names(path);
		for (auto const &gr : region_names)
		{
			std::string p = path + "/" + gr + "/Nodes";
			m_data[gr] = internal::cfs_data_reader<scalar_t>::read(file, p);
		}
	}

	matrix_t const &get_data(std::string const &s) const { return m_data.at(s); }

private:
	std::map<std::string, matrix_t> m_data;
};

}

#endif
