#ifndef CFS_MESH_REGION_H_INCLUDED
#define CFS_MESH_REGION_H_INCLUDED

#include <iostream>
#include <string>

#include "h5_file.hpp"

#include <Eigen/Dense>

namespace NiHu
{

class cfs_mesh_region
{
public:
	typedef Eigen::Matrix<int, Eigen::Dynamic, 1> indices_t;
	typedef unsigned int dimension_t;

	cfs_mesh_region() = default;

	cfs_mesh_region(h5_file &file, std::string const& path);

	dimension_t get_dimension() const { return m_dimension; }

	indices_t const &get_elements() const { return m_elements; }
	indices_t const &get_nodes() const { return m_nodes; }

	/**
	 * @brief Read the region from a HDF5 file
	 * @param[in] file HDF5 file to read from
	 * @param[in] path Path of the region in the HDF
	 */
	void read(h5_file &file, std::string const &path);

	/**
	 * @brief Print for debugging
	 * @param[in] os Output stream to print to
	 */
	void print_debug(std::ostream &os = std::cout, std::string const &prefix = "");

private:
	indices_t m_nodes;
	indices_t m_elements;
	dimension_t m_dimension;
};

}

#endif
