#include "dom_elem.h"

#include <util/XMLString.hpp>
#include <dom/DOMNodeList.hpp>

#include <stdexcept>
#include <algorithm>

dom_elem dom_elem::get_child(std::string const &tag) const
{
	xercesc::DOMElement *res = nullptr;

	XMLCh *tagname = xercesc::XMLString::transcode(tag.c_str());
	xercesc::DOMNodeList *children = m_dom_element->getChildNodes();

	size_t length = children->getLength();
	for (size_t i = 0; i < length; ++i)
	{
		xercesc::DOMElement *p = dynamic_cast<xercesc::DOMElement *>(children->item(i));
		if (p && xercesc::XMLString::compareString(p->getTagName(), tagname) == 0)
		{
			res = p;
			break;
		}
	}

	xercesc::XMLString::release(&tagname);

	return dom_elem(res);
}

std::string dom_elem::get_attribute(std::string const &name) const
{
	XMLCh *xname = xercesc::XMLString::transcode(name.c_str());
	XMLCh const *xattr = m_dom_element->getAttribute(xname);
	xercesc::XMLString::release(&xname);
	return xmlstr2stdstr(xattr);
}

std::string dom_elem::get_attribute(std::string const &path, std::string const &name) const
{
	auto start = path.begin();
	dom_elem sub_elem = *this;
	std::string::const_iterator s;
	while ((s = std::find(start, path.end(), '/')) != path.end())
	{
		sub_elem = sub_elem.get_child(std::string(start, s));
		if (!sub_elem)
			throw std::runtime_error("Path " + path + " not found");
		start = s + 1;
	}
	sub_elem = sub_elem.get_child(std::string(start, path.end()));
	if (!sub_elem)
		throw std::runtime_error("Path " + path + " not found");
	return sub_elem.get_attribute(name);
}

std::vector<dom_elem> dom_elem::get_children(std::string const &tag) const
{
	std::vector<dom_elem> res;

	XMLCh *tagname = xercesc::XMLString::transcode(tag.c_str());
	xercesc::DOMNodeList *children = m_dom_element->getChildNodes();

	size_t length = children->getLength();
	for (size_t i = 0; i < length; ++i)
	{
		xercesc::DOMElement *p = dynamic_cast<xercesc::DOMElement *>(children->item(i));
		if (p && xercesc::XMLString::compareString(p->getTagName(), tagname) == 0)
			res.push_back(dom_elem(p));
	}

	xercesc::XMLString::release(&tagname);

	return res;
}
