#include "h5_file.hpp"

namespace NiHu
{

	// H5::PredType const h5_type_traits<int>::h5_pred_type = H5::PredType::NATIVE_INT;
	// H5::PredType const h5_type_traits<double>::h5_pred_type = H5::PredType::NATIVE_DOUBLE;

	std::vector<std::string> h5_file::get_subgroup_names(std::string const &path)
	{
		static const int MAX_NAME = 256;
		static char name_buff[MAX_NAME];

		if (!exists(path))
			throw std::runtime_error("h5_file::get_subgroups: group not found ('" + path + "')");

		H5::Group regions_group = m_file.openGroup(path.c_str());

		hsize_t num_obj;

		std::vector<std::string> res;

		if (H5Gget_num_objs(regions_group.getId(), &num_obj) < 0)
			throw std::runtime_error("Failed to read number of objects from /Mesh/Regions");
		for (hsize_t i = 0; i < num_obj; i++)
		{
			H5G_obj_t obj_type;
			// Find out the type of the member
			if ((obj_type = H5Gget_objtype_by_idx(regions_group.getId(), i)) < 0)
				throw std::runtime_error("Failed to get object type for object #" + std::to_string(i) + " in /Mesh/Regions");
			// If the type is a group, it must be a region
			if (obj_type == H5G_GROUP)
			{
				// Read the name of the region
				if (H5Gget_objname_by_idx(regions_group.getId(), i, name_buff, MAX_NAME) < 0)
					throw std::runtime_error("Failed to get object type for object #" + std::to_string(i) + " in /Mesh/Regions");
				// Read the region from the file
				std::string name(name_buff);
				res.push_back(name);
			}
		}

		return res;
	}

template<>
std::string h5_file::read_attribute<std::string>(std::string const &path, std::string const &name)
{
    H5::Group group = m_file.openGroup(path);
    H5::Attribute attr = group.openAttribute(name);

    std::string value;

    H5::DataType type = attr.getDataType();
    attr.read(type, value);

    return value;
}



} // of namespace NiHu
