#ifndef XML_PARSER_H
#define XML_PARSER_H

#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMDocumentType.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMNodeIterator.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMText.hpp>

#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/XMLUni.hpp>

#include <stdexcept>
#include <string>
#include <vector>

#include "acoustic_material.h"

enum
{
    ERROR_ARGS = 1,
    ERROR_XERCES_INIT,
    ERROR_PARSE,
    ERROR_EMPTY_DOCUMENT
};

class xml_parser
{
public:
    xml_parser();

    ~xml_parser();
    
    void readConfigFile(std::string const &filename);

    std::vector<acoustic_material> readMaterialFile(std::string const &filename);

    char *getOptionA() { return m_OptionA; };

    char *getOptionB() { return m_OptionB; };

    static std::string xmlstr2stdstr(XMLCh const *xmlstr);

    static xercesc::DOMElement *getTagByName(xercesc::DOMElement *parent, std::string const &tagname);

    static std::string getAttribute(xercesc::DOMElement *parent, std::string const &attrname);


private:
    xercesc::XercesDOMParser *m_ConfigFileParser;
    char *m_OptionA;
    char *m_OptionB;

    // Internal class use only. Hold Xerces data in UTF-16 SMLCh type.

    XMLCh *TAG_root;

    XMLCh *TAG_ApplicationSettings;
    XMLCh *ATTR_OptionA;
    XMLCh *ATTR_OptionB;
};


#endif
