#include "xml_file.h"

#include <stdexcept>

#include "util/PlatformUtils.hpp"
#include "util/XMLException.hpp"
#include "util/XMLString.hpp"

void xml_file::initialize()
{
	try
	{
		xercesc::XMLPlatformUtils::Initialize();
	}
	catch (xercesc::XMLException &e)
	{
		char *message = xercesc::XMLString::transcode(e.getMessage());
		std::string msg(message);
		xercesc::XMLString::release(&message);
		throw std::runtime_error(msg);
	}
}

void xml_file::terminate()
{
	try
	{
		xercesc::XMLPlatformUtils::Terminate();
	}
	catch (xercesc::XMLException &e)
	{
		char *message = xercesc::XMLString::transcode(e.getMessage());
		std::string msg(message);
		xercesc::XMLString::release(&message);
		throw std::runtime_error(msg);
	}
}

xml_file::xml_file(std::string const &fname)
	: m_dom_parser(nullptr)
{
	m_dom_parser = new xercesc::XercesDOMParser;
	m_dom_parser->setValidationScheme(xercesc::XercesDOMParser::Val_Never);
	m_dom_parser->setDoNamespaces(false);
	m_dom_parser->setDoSchema(false);
	m_dom_parser->setLoadExternalDTD(false);
	try
	{
		m_dom_parser->parse(fname.c_str());
	}
	catch (xercesc::XMLException &e)
	{
		char *message = xercesc::XMLString::transcode(e.getMessage());
		std::string msg(message);
		xercesc::XMLString::release(&message);
		delete m_dom_parser;
		m_dom_parser = nullptr;
		throw std::runtime_error("Error parsing file " + msg);
	}
}

void xml_file::close()
{
	delete m_dom_parser;
	m_dom_parser = nullptr;
}

xml_file::~xml_file()
{
	close();
}

dom_elem xml_file::get_root() const
{
	return dom_elem(m_dom_parser->getDocument()->getDocumentElement());
}
